# Overview 

[![Build Status](https://drone.nitowa.xyz/api/badges/npm-packages/xrpio/status.svg)](https://drone.nitowa.xyz/npm-packages/xrpio) 
[![Current Version](https://img.shields.io/npm/v/xrpio.svg)](https://www.npmjs.com/package/xrpio)
[![Weekly Downloads](https://img.shields.io/npm/dw/xrpio?color=important)](https://www.npmjs.com/package/xrpio)
[![License Type](https://img.shields.io/npm/l/xrpio?color=blueviolet)](https://gitea.nitowa.xyz/docs/xrpio/src/branch/master/LICENSE.md)

xrpio is a library that allows you to write and read arbitrary data in the ripple blockchain.

# How to install
```
npm i xrpio
```

# How it works

Transactions on the ripple blockchain are allowed to carry up to 1kB of arbitrary data via the memo field. 
We can use this to store data of any size by building a tree of references between these transactions that can then be reassembled by reading them back from the blockchain.
In order to generate these transactions xrpio sends payments with the minimum denomination between two wallets controlled by the user. 

xrpio automatically takes care of the logistics behind this technique as well as compression of the data.

Highly simplified, you can visualize the process like this:

![xrpio treewrite](https://i.imgur.com/G2HofSE.gif)

In practice each node does of course store significantly more data.


# Caution
This library is in an early stage of development and **breaking changes may occur spontaneously and without regard of semantic versioning until the v1.0.0 release**.

## <span style="color:red"> Operation on the main-net is untested and should not be used in production! If you want to deploy this library with the main-net please download the sources and modify them to your needs.</span>
<br />

# Quickstart
```typescript
import { xrpIO } from "xrpio"

const api = new xrpIO("wss://some_ripple_node.net:51233")
await api.connect()

txHash = await api.treeWrite("arbitrary text 123", receiveWallet.address, sendWallet.secret)
data = await api.treeRead([txHash])

console.log(data) //"arbitrary text 123"
```

# Known Bugs
- When using large data (>300kB) and public nodes, the writing process may fail due to rate limits. xrpio will attempt to mitigate this, but success is inconsistent.

# [Full documentation](https://gitea.nitowa.xyz/docs/xrpio)
